##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

"""
$Id: interfaces.py 81877 2007-11-16 04:16:46Z rogerineichen $
"""
__docformat__ = 'restructuredtext'

import zope.interface
from zope.schema.interfaces import IVocabularyTokenized

from z3c.i18n import MessageFactory as _


class IReadI18n(zope.interface.Interface):
    """Let the language switch to the desired language.
    
    Support add and remove objects of a language.
    """

    def getDefaultLanguage():
        """Return the default language."""

    def getPreferedLanguage():
        """Return the best matching language."""

    def getAvailableLanguages():
        """Find all the languages that are available."""

    def getAttribute(name, language=None):
        """Get name attribute of the language specific translation.

        Parameter:

        name -- Attribute name.

        language -- Language code for example 'de'. If None the default language
        is returned.

        Return Value:

        object -- Evaluate of the language specific data object.

        Exceptions:

        KeyError -- If attribute or language code does not exists.

        """

    def queryAttribute(name, language=None, default=None):
        """Get name attribute of the language specific translation or default.

        Parameter:

        name -- Attribute name.

        language -- Language code for example 'de'. If None the default language
        is returned.

        default -- Any object.

        Return Value:

        object -- Evaluate of the language specific data object or return default
        if not found.

        """


class IWriteI18n(zope.interface.Interface):
    """Let the language switch to the desired language.
    
    Support add and remove objects of a language.
    """

    def setDefaultLanguage(language):
        """Set the default language, which will be used if the language is not
        specified, or not available.
        """

    def addLanguage(language, *args, **kw):
        """Add a i18n base object of the i18n type. The ``*args``, ``**kw`` 
        can be used for the constructor of a new sub object.
        """

    def removeLanguage(language):
        """Remove the object under the given language."""

    def setAttributes(language, **kws):
        """Set the language specific attribute of the translation defined by kw.

        Parameter:

        language -- Language code for example ``'de'``

        kws -- Attributes that have to be set as keyword value pairs.

        Exceptions:

        KeyError -- If attribute does not exists.
        
        """


class II18n(IReadI18n, IWriteI18n):
    """Read and write support for I18n objects."""


class II18nLanguageSwitch(zope.interface.Interface):
    """Let the language switch to the desired language."""

    def getLanguage():
        """Returns the used language."""

    def setLanguage(language):
        """Sets the language for useing in the adapter."""


class IAvailableLanguages(zope.interface.Interface):

    def getAvailableLanguages():
        """Returns a list of available languages if we provide IReadI18n."""

    def hasAvailableLanguages():
        """View for to check if we have i18n support on a context."""


class IAvailableLanguagesVocabulary(IVocabularyTokenized):
    """Available languages."""




##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

"""
$Id: testing.py 81877 2007-11-16 04:16:46Z rogerineichen $
"""
__docformat__ = 'restructuredtext'

import unittest
import zope.interface
from zope.interface.verify import verifyClass
from zope.app.testing import placelesssetup

from z3c.language.switch import IReadI18n
from z3c.language.switch import IWriteI18n
from z3c.language.switch import II18n
from z3c.language.switch import II18nLanguageSwitch
from z3c.testing import InterfaceBaseTest
from z3c.testing import marker_pos
from z3c.testing import marker_kws

def sorted(list):
    list.sort()
    return list


################################################################################
#
# Public Test implementations
#
################################################################################

class IContentObject(zope.interface.Interface):
    """IContentObject interface."""


class II18nContentObject(zope.interface.Interface):
    """II18nContentObject interface."""


class ContentObject(object):
    """Content type."""

    zope.interface.implements(IContentObject)

    __parent__ = __name__ = None
    
    _title = u''

    def __init__(self, title=''):
        self._title = title

    def getTitle(self):
        """Get the title of the object."""
        return self._title

    def setTitle(self, title):
        """Set the title of the object."""
        self._title = title

    title = property(getTitle, setTitle)


class I18nContentObject(object):
    """i18n content type."""

    zope.interface.implements(II18nContentObject, IReadI18n, IWriteI18n)

    def __init__(self, title='', defaultLanguage=None):
        self._data = {}
        if defaultLanguage == None:
            self._defaultLanguage = 'en'
        else:
            self._defaultLanguage = defaultLanguage
        document = self._create(title)
        self._data[self._defaultLanguage] = document

    def getTitle(self, language=None):
        """Get the title of the object."""
        document = self._get_or_add(language)
        return document.title

    def setTitle(self, title, language=None):
        """Set the title of the object."""
        document = self._get_or_add(language)
        document.title = title

    title = property(getTitle, setTitle)

    # IReadI18n interface
    def getAvailableLanguages(self):
        """See `z3c.langauge.switch.interfaces.IReadI18n`"""
        keys = self._data.keys()
        keys.sort()
        return keys

    def getDefaultLanguage(self):
        """See `z3c.langauge.switch.interfaces.IReadI18n`"""
        return self._defaultLanguage

    # IWriteI18n interface
    def setDefaultLanguage(self, language):
        """See `z3c.langauge.switch.interfaces.IWriteI18n`"""
        if language not in self._data:
            raise ValueError(
                  'cannot set nonexistent language (%s) as default' % language)
        self._defaultLanguage = language

    def addLanguage(self, language, title=''):
        """See `z3c.langauge.switch.interfaces.IWriteI18n`"""
        return self._get_or_add(language, title)

    def removeLanguage(self, language):
        """See `z3c.langauge.switch.interfaces.IWriteI18n`"""
        if language == self.getDefaultLanguage():
            raise ValueError('cannot remove default language (%s)' % language)
        elif language not in self._data:
            raise ValueError('cannot remove nonexistent language (%s)' 
                % language)
        else:
            del self._data[language]
            self._p_changed = True

    # helper methods
    def _create(self, title=''):
        """Create a new subobject of the type document."""
        return ContentObject(title)

    def _get(self, language):
        """Helper function -- return a subobject for a given language,
        and if it does not exist, return a subobject for the default
        language.
        """
        document = self._data.get(language)
        if not document:
            document = self._data[self.getDefaultLanguage()]
        return document

    def _get_or_add(self, language, title=''):
        """Helper function -- return a subobject for a given language,
        and if it does not exist, create and return a new subobject.
        """
        language = self._getLang(language)
        document = self._data.get(language)
        if not document:
            document = self._create(title)
            self._data[language] = document
            self._p_changed = 1
        return document

    def _getLang(self, language):
        """Returns the given language or the default language."""
        if language == None:
            language = self.getDefaultLanguage()

        return language


class I18nContentObjectLanguageSwitch(object):
    """Language switch for I18nContentObject."""
    
    zope.interface.implements(II18nLanguageSwitch, II18nContentObject)

    _language = 'en'

    def __init__(self, context):
        self.context = context
        self._language = context.getDefaultLanguage()

    # II18nLanguageSwitch interface
    def getLanguage(self):
        """See `z3c.langauge.switch.interfaces.II18nLanguageSwitch`"""
        return self._language

    def setLanguage(self, language):
        """See `z3c.langauge.switch.interfaces.II18nLanguageSwitch`"""
        self._language = language

    # II18nContentObject interface
    def getTitle(self):
        """Get the title of the object."""
        return self.context.getTitle(self.getLanguage())

    def setTitle(self, title):
        """Set the title of the object."""
        self.context.setTitle(title, self.getLanguage())

    title = property(getTitle, setTitle)


################################################################################
#
# Public Base Tests
#
################################################################################

class BaseTestII18n(InterfaceBaseTest):

    def getTestInterface(self):
        raise NotImplementedError, \
            'Subclasses has to implement getTestInterface()'

    def makeI18nTestObject(self, **kws):
        kws['defaultLanguage'] = kws.get('defaultLanguage', 'de')
        return self.makeTestObject(**kws)

    # IReadI18n tests
    def test_getAvailableLanguages(self):
        i18n = self.makeI18nTestObject()
        res = ['de']
        self.assertEqual(sorted(i18n.getAvailableLanguages()), res)

    def test_getDefaultLanguage(self):
        i18n = self.makeI18nTestObject()
        self.assertEqual(i18n.getDefaultLanguage(), 'de')

    # IWriteI18n tests
    def test_setDefaultLanguage(self):
        i18n = self.makeI18nTestObject()
        self.assertEqual(i18n.getDefaultLanguage(), 'de')
        i18n.addLanguage('en')
        i18n.setDefaultLanguage('en')
        self.assertEqual(i18n.getDefaultLanguage(), 'en')

    def test_addLanguage(self):
        i18n = self.makeI18nTestObject()
        i18n.addLanguage('at')
        res = ['at', 'de']
        self.assertEqual(sorted(i18n.getAvailableLanguages()), res)

    def test_removeLanguage(self):
        i18n = self.makeI18nTestObject()
        i18n.addLanguage('fr')
        res = ['de', 'fr']
        self.assertEqual(sorted(i18n.getAvailableLanguages()), res)
        i18n.removeLanguage('fr')
        res = ['de']
        self.assertEqual(sorted(i18n.getAvailableLanguages()), res)
        self.assertRaises(ValueError, i18n.removeLanguage, 'de')
        self.assertRaises(ValueError, i18n.removeLanguage, 'undefined')

    def test_II18n_Interface(self):
        i18n = self.makeI18nTestObject()
        class_ = self.getTestClass()
        self.failUnless(IReadI18n.implementedBy(class_))
        self.failUnless(IWriteI18n.implementedBy(class_))
        self.failUnless(II18n.implementedBy(class_))
        self.failUnless(verifyClass(IReadI18n, class_))
        self.failUnless(verifyClass(IWriteI18n, class_))
        self.failUnless(verifyClass(II18n, class_))


class BaseTestI18nLanguageSwitch(InterfaceBaseTest):

    def getTestClass(self):
        raise NotImplementedError, \
            'Subclasses has to implement getTestClass()'

    def getTestInterface(self):
        raise NotImplementedError, \
            'Subclasses has to implement getTestInterface()'

    def getAdaptedClass(self):
        raise NotImplementedError, \
            'Subclasses has to implement getAdaptedClass()'
    
    def makeTestObject(self, object=None, *pos, **kws):
        # provide default positional or keyword arguments
        if self.getTestPos() is not marker_pos and not pos:
            pos = self.getTestPos()

        if self.getTestKws() is not marker_kws and not kws:
            kws = self.getTestKws()
       
        testclass = self.getAdaptedClass()
        iface = self.getTestInterface()
        
        if object is None:
            # a class instance itself is the object to be tested.         
            obj = testclass(*pos, **kws)
            return iface(obj)

        else:
            # an adapted instance is the object to be tested. 
            obj = testclass(object, *pos, **kws)
            return iface(obj)

    def setUp(self):
        placelesssetup.setUp()
        factory = self.getTestClass()
        iface = self.getTestInterface()
        required = self.getAdaptedClass()
        # register language switch for test interface adapter
        zope.component.provideAdapter(factory, (required,), iface)

    def tearDown(self):
        placelesssetup.tearDown()

    def test_getLanguage(self):
        obj = self.makeTestObject()
        self.assertEqual(obj.getLanguage(), 'de')

    def test_setLanguage(self):
        obj = self.makeTestObject()
        obj.setLanguage('fr')
        self.assertEqual(obj.getLanguage(), 'fr')

    def test_i18n_II18nLanguageSwitch_Interface(self):
        class_ = self.getTestClass()
        obj = self.makeTestObject()
        iface = self.getTestInterface()
        adapter = iface(obj)
        self.failUnless(II18nLanguageSwitch.implementedBy(class_))
        self.failUnless(verifyClass(II18nLanguageSwitch, class_))
        self.failUnless(iface.providedBy(adapter))



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

"""
$Id: app.py 69743 2006-08-24 00:05:38Z rogerineichen $
"""
__docformat__ = 'restructuredtext'

import persistent
import zope.interface
import zope.component
import zope.event
import zope.lifecycleevent
from zope.i18n.interfaces import INegotiator
from zope.security.interfaces import NoInteraction
from zope.security.management import getInteraction

from z3c.language.switch import II18n


def getRequest():
    try:
        interaction = getInteraction()
        request = interaction.participations[0]
    except NoInteraction:
        request = None
    except IndexError:
        request = None
    return request


class I18n(persistent.Persistent, object):
    """Mixin implementation of II18n.
    
    Set a factory class in our implementation. The _create method will initalize 
    this class and use it as a child object providing the attributes without 
    languages.

    You can use this class as mixin for i18n implementations:

    >>> from z3c.language.switch.app import I18n
    >>> class Person(object):
    ...     def __init__(self, firstname, lastname):
    ...         self.firstname = firstname
    ...         self.lastname = lastname

    >>> class I18nPerson(I18n):
    ...     _defaultLanguage = 'en'
    ...     _factory = Person

    Now you can initialize the default translation:

    >>> i18n = I18nPerson(firstname='Bob', lastname='Miller')
    >>> i18n.getDefaultLanguage()
    'en'
    >>> i18n.getPreferedLanguage()
    'en'

    You can access the attributes of the default translation:

    >>> i18n.getAttribute('firstname')
    'Bob'
    >>> i18n.getAttribute('lastname')
    'Miller'
    
    That is the same as accessing the attribute using a language parameter:

    >>> i18n.getAttribute('firstname', 'en')
    'Bob'

    If an attribute is not available a AttributeError is raised. The 
    queryAttribute method offers a save way for unsecure access:

    >>> i18n.getAttribute('name')
    Traceback (most recent call last):
    ...
    AttributeError: 'Person' object has no attribute 'name'

    >>> i18n.queryAttribute('name') is None
    True

    You can set the attributes an other time:

    >>> i18n.setAttributes('en', firstname='Foo', lastname='Bar')
    >>> i18n.getAttribute('firstname')
    'Foo'
    >>> i18n.getAttribute('lastname')
    'Bar'

    You can initialize the default translation using a specific language:

    >>> i18n = I18nPerson(defaultLanguage='fr', firstname='Robert', 
    ...                   lastname='Moulin')
    >>> i18n.getDefaultLanguage()
    'fr'
    >>> i18n.getPreferedLanguage()
    'fr'

    """

    _data = None
    # sublclasses should overwrite this attributes.
    _defaultLanguage = None
    _factory = None

    # private method (subclasses might overwrite this method)
    def _defaultArgs(self):
        return None

    zope.interface.implements(II18n)


    def __init__(self, defaultLanguage=None, *args, **kws):
        # preconditions
        if self._defaultLanguage is None:
            raise NotImplementedError('_defaultLanguage')

        if self._factory is None:
            raise NotImplementedError('_factory')

        # essentials
        if defaultLanguage is None:
            defaultLanguage = self._defaultLanguage

        # initialize data store for translations
        self._setDataOnce()

        # initialize default translation
        self._get_or_add(defaultLanguage, *args, **kws)

        # set default language
        self.setDefaultLanguage(defaultLanguage)

    # private method
    def _setDataOnce(self):
        if self._data is None:
            self._data = {}

    # private method: access self._data only using this method
    def _getData(self):
        return self._data
        
    # z3c.langauge.switch.IReadI18n
    def getAvailableLanguages(self):
        """See `z3c.langauge.switch.interfaces.IReadI18n`"""
        keys = self._getData().keys()
        keys.sort()
        return keys

    def getDefaultLanguage(self):
        """See `z3c.langauge.switch.interfaces.IReadI18n`"""
        return self._defaultLanguage

    def getPreferedLanguage(self):
        # evaluate the negotiator
        language = None
        request = getRequest()
        negotiator = None
        try:
            negotiator = zope.component.queryUtility(INegotiator, 
                name='', context=self)
        except zope.component.ComponentLookupError:
            # can happens during tests without a site and sitemanager
            pass
        if request and negotiator:
            language = negotiator.getLanguage(self.getAvailableLanguages(), request)
        if language is None:
            language = self.getDefaultLanguage()
        if language is None:
            # fallback language for functional tests, there we have a cookie request
            language = 'en'
        return language

    def getAttribute(self, name, language=None):
        # preconditions
        if language is None:
            language = self.getDefaultLanguage()

        if not language in self.getAvailableLanguages():
            raise KeyError(language)

        # essentials
        data = self._getData()[language]
        return getattr(data, name)
        
    def queryAttribute(self, name, language=None, default=None):
        try:
            return self.getAttribute(name, language)
        except AttributeError:
            return default

    # z3c.langauge.switch.IReadI18n
    def setDefaultLanguage(self, language):
        """See `z3c.langauge.switch.interfaces.IWriteI18n`"""
        data = self._getData()
        if language not in data:
            raise ValueError(
                'cannot set nonexistent language (%s) as default' % language)
        self._defaultLanguage = language

    def addLanguage(self, language, *args, **kw):
        """See `z3c.langauge.switch.interfaces.IWriteI18n`"""
        if not args and not kw:
            if self._defaultArgs() is not None:
                args = self._defaultArgs()

        self._get_or_add(language, *args, **kw)
        zope.event.notify(zope.lifecycleevent.ObjectModifiedEvent(self))

    def removeLanguage(self, language):
        """See `z3c.langauge.switch.interfaces.IWriteI18n`"""
        data = self._getData()
        if language == self.getDefaultLanguage():
            raise ValueError('cannot remove default language (%s)' % language)
        elif language not in data:
            raise ValueError('cannot remove nonexistent language (%s)' 
                % language)
        else:
            del data[language]
            self._p_changed = True
        zope.event.notify(zope.lifecycleevent.ObjectModifiedEvent(self))

    def setAttributes(self, language, **kws):
        # preconditions
        if not language in self.getAvailableLanguages():
            raise KeyError(language)

        data = self._getData()
        obj = data[language]
        
        for key in kws:
            if not hasattr(obj, key):
                raise KeyError(key)

        # essentials
        for key in kws:
            setattr(obj, key, kws[key])
        else:
            self._p_changed = True           
        zope.event.notify(zope.lifecycleevent.ObjectModifiedEvent(self))

    # private helper methods
    def _create(self, *args, **kw):
        """Create a new subobject of the type document."""
        factory = self._factory
        obj = factory(*args, **kw)
        zope.event.notify(zope.lifecycleevent.ObjectCreatedEvent(obj))
        return obj

    def _get(self, language):
        """Helper function -- return a subobject for a given language,
        and if it does not exist, return a subobject for the default
        language.
        """
        data = self._getData()
        obj = data.get(language, None)
        if obj is None:
            obj = data[self.getDefaultLanguage()]
        return obj

    def _get_or_add(self, language, *args, **kw):
        """Helper function -- return a subobject for a given language,
        and if it does not exist, create and return a new subobject.
        """
        data = self._getData()
        language = self._getLang(language)
        obj = data.get(language, None)
        if obj is None:
            obj = self._create(*args, **kw)
            data[language] = obj
            # this (ILocation info) is needed for the pickler used by the
            # locationCopy method in the ObjectCopier class
            obj.__parent__ = self
            obj.__name__ = language
            self._p_changed = 1
        return obj

    def _getLang(self, language):
        """Returns the given language or the default language."""
        if language == None:
            language = self.getDefaultLanguage()

        return language



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

"""
$Id: tests.py 69742 2006-08-23 22:17:00Z rogerineichen $
"""
__docformat__ = 'restructuredtext'

import doctest
import unittest

from zope.testing import doctestunit


def test_suite():
    return unittest.TestSuite((
        doctestunit.DocFileSuite('README.txt'),
        doctestunit.DocFileSuite('app.py')
        ))

if __name__=='__main__':
    unittest.main(defaultTest='test_suite')



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

"""
$Id: adapters.py 69742 2006-08-23 22:17:00Z rogerineichen $
"""
__docformat__ = 'restructuredtext'

import zope.interface

from z3c.language.switch import II18n
from z3c.language.switch import II18nLanguageSwitch


class I18nLanguageSwitch(object):
    """Mixing class for switch a language on a object.
    
    Use the method _getI18n for set the i18n object.
    """
    zope.interface.implements(II18nLanguageSwitch)

    _language = 'en'

    def __init__(self, context):
        self.context = context
        self.i18n = self._getI18n()
        self._language = self._getDefaultLanguage()

    def _getDefaultLanguage(self):
        """Subclasses may overwrite this method."""
        return self.i18n.getDefaultLanguage()

    def _getI18n(self):
        """Subclasses may overwrite this method."""
        return self.context

    # II18nLanguageSwitch interface
    def getLanguage(self):
        """See `z3c.langauge.switch.interfaces.II18nLanguageSwitch`"""
        return self._language

    def setLanguage(self, language):
        """See `z3c.langauge.switch.interfaces.II18nLanguageSwitch`"""
        self._language = language


class I18nAdapter(object):
    """Mixing class for i18n adapters which must provide the adapted object 
       under the attribute 'self.i18n'.
    """
    zope.interface.implements(II18n)

    # z3c.langauge.switch.IReadI18n
    def getAvailableLanguages(self):
        """See `z3c.langauge.switch.interfaces.IReadI18n`"""
        return self.i18n.getAvailableLanguages()

    def getDefaultLanguage(self):
        """See `z3c.langauge.switch.interfaces.IReadI18n`"""
        return self.i18n.getDefaultLanguage()

    def getPreferedLanguage(self):
        """See `z3c.langauge.switch.interfaces.IReadI18n`"""
        return self.i18n.getPreferedLanguage()

    def getAttribute(self, name, language=None):
        return self.i18n.getAttribute(name, language)
        
    def queryAttribute(self, name, language=None, default=None):
        return self.i18n.queryAttribute(name, language, default)

    # z3c.langauge.switch.IWriteI18n interface
    def setDefaultLanguage(self, language):
        """See `z3c.langauge.switch.interfaces.IWriteI18n`"""
        return self.i18n.setDefaultLanguage(language)

    def addLanguage(self, language, *args, **kw):
        """See `z3c.langauge.switch.interfaces.IWriteI18n`"""
        return self.i18n.addLanguage(language, *args, **kw)

    def removeLanguage(self, language):
        """See `z3c.langauge.switch.interfaces.IWriteI18n`"""
        self.i18n.removeLanguage(language)

    def setAttributes(self, language, **kws):
        """See `z3c.langauge.switch.interfaces.IWriteI18n`"""
        self.i18n.setAttributes(language, **kws)



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

"""
$Id: views.py 81877 2007-11-16 04:16:46Z rogerineichen $
"""
__docformat__ = 'restructuredtext'

import zope.interface
from zope.publisher.browser import BrowserView

from z3c.language.switch import IReadI18n
from z3c.language.switch import IAvailableLanguages


class ContentView(BrowserView):

    zope.interface.implements(IAvailableLanguages)

    def getAvailableLanguages(self):
        """Returns a list of available languages if we provide IReadI18n."""

        if IReadI18n.providedBy(self.context):
            return self.context.getAvailableLanguages()
        else:
            return []


    def hasAvailableLanguages(self):
        """View for to check if we have i18n support on a context."""

        if IReadI18n.providedBy(self.context):
            return True
        else:
            return False



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

"""
$Id: vocabulary.py 69742 2006-08-23 22:17:00Z rogerineichen $
"""
__docformat__ = 'restructuredtext'

import zope.interface

from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

from z3c.language.switch import IAvailableLanguagesVocabulary


class AvailableLanguagesVocabulary(SimpleVocabulary):
    """A vocabular of available languages from the context object."""

    zope.interface.implements(IAvailableLanguagesVocabulary)

    zope.interface.classProvides(IVocabularyFactory)

    def __init__(self, context):
        terms = []
        
        # returns available languages form the object itself
        # but just after creation of the object
        try:
            languages = context.getAvailableLanguages()
        except AttrbiteError:
            languages = []

        for lang in languages:
            terms.append(SimpleTerm(lang, lang, lang))

        terms.sort(lambda lhs, rhs: cmp(lhs.title, rhs.title))
        super(AvailableLanguagesVocabulary, self).__init__(terms)



##############################################################################
#
# Copyright (c) 2005 Zope Foundation and Contributors.
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#
##############################################################################

"""
$Id: property.py 69742 2006-08-23 22:17:00Z rogerineichen $
"""
__docformat__ = 'restructuredtext'

_marker = object()


class I18nFieldProperty(object):
    """Computed attributes based on schema fields and i18n implementation.

    Field properties provide default values, data validation and error messages
    based on data found in field meta-data.

    Note that I18nFieldProperties can only be used for attributes stored in
    a translation object. The class using this I18nFieldProperty must implement
    z3c.langauge.switch.II18n.
    """

    def __init__(self, field, name=None):
        if name is None:
            name = field.__name__

        self.__field = field
        self.__name = name

    def __get__(self, inst, klass):
        if inst is None:
            return self

        value = inst.queryAttribute(self.__name, inst.getPreferedLanguage(), _marker)
        if value is _marker:
            field = self.__field.bind(inst)
            value = getattr(field, 'default', _marker)
            if value is _marker:
                raise AttributeError, self.__name

        return value

    def __set__(self, inst, value):
        field = self.__field.bind(inst)
        field.validate(value)
        # make kws dict
        kws = {}
        kws[self.__name] = value
        inst.setAttributes(inst.getPreferedLanguage(), **kws)

    def __getattr__(self, name):
        return getattr(self.__field, name)


class I18nLanguageSwitchFieldProperty(object):
    """Computed attributes based on schema fields and i18n language switch implementation.

    Field properties provide default values, data validation and error messages
    based on data found in field meta-data.

    Note that I18nLanguageSwitchFieldProperty can only be used for attributes stored in
    self.i18n.
    
    If you use this field in simply object, there is a adapter called 
    'I18nLanguageSwitch' where the context attribute is setting to i18n. 
    """

    def __init__(self, field, name=None):
        if name is None:
            name = field.__name__

        self.__field = field
        self.__name = name

    def __get__(self, inst, klass):
        # essentails
        if inst is None:
            return self
        i18n = inst.i18n
        lang = inst.getLanguage()

        value = i18n.queryAttribute(self.__name, lang, _marker)
        if value is _marker:
            field = self.__field.bind(inst)
            value = getattr(field, 'default', _marker)
            if value is _marker:
                raise AttributeError, self.__name

        return value

    def __set__(self, inst, value):
        # essentails
        i18n = inst.i18n
        lang = inst.getLanguage()

        field = self.__field.bind(inst)
        field.validate(value)
        # make kws dict
        kws = {}
        kws[self.__name] = value
        i18n.setAttributes(lang, **kws)

    def __getattr__(self, name):
        return getattr(self.__field, name)
